var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../core/AtomComponent", "../core/AtomUri", "../core/FormattedString", "../core/types", "./ReferenceService"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.NavigationService = exports.NotifyType = void 0;
    var AtomComponent_1 = require("../core/AtomComponent");
    var AtomUri_1 = require("../core/AtomUri");
    var FormattedString_1 = require("../core/FormattedString");
    var types_1 = require("../core/types");
    var ReferenceService_1 = require("./ReferenceService");
    // export interface ILocation {
    //     href?: string;
    //     hash?: INameValues;
    //     host?: string;
    //     hostName?: string;
    //     port?: string;
    //     protocol?: string;
    // }
    var NotifyType;
    (function (NotifyType) {
        NotifyType["Information"] = "info";
        NotifyType["Warning"] = "warn";
        NotifyType["Error"] = "error";
    })(NotifyType = exports.NotifyType || (exports.NotifyType = {}));
    var nameSymbol = UMD.nameSymbol;
    function hasPageUrl(target) {
        var url = target[nameSymbol];
        if (!url) {
            return false;
        }
        var baseClass = Object.getPrototypeOf(target);
        if (!baseClass) {
            // this is not possible...
            return false;
        }
        return baseClass[nameSymbol] !== url;
    }
    var NavigationService = /** @class */ (function () {
        function NavigationService(app) {
            this.app = app;
            this.callbacks = [];
        }
        /**
         *
         * @param pageName url
         * @param p parameters
         * @param options {@link IPageOptions}
         */
        NavigationService.prototype.openPage = function (pageName, p, options) {
            options = options || {};
            if (typeof pageName !== "string") {
                if (hasPageUrl(pageName)) {
                    pageName = pageName[nameSymbol];
                }
                else {
                    var rs = this.app.resolve(ReferenceService_1.default);
                    var host = pageName instanceof AtomComponent_1.AtomComponent ? "reference" : "class";
                    var r = rs.put(pageName);
                    pageName = "ref://" + host + "/" + r.key;
                }
            }
            var url = new AtomUri_1.AtomUri(pageName);
            if (p) {
                for (var key in p) {
                    if (p.hasOwnProperty(key)) {
                        var element = p[key];
                        if (element === undefined) {
                            continue;
                        }
                        if (element === null) {
                            url.query["json:" + key] = "null";
                            continue;
                        }
                        if (key.startsWith("ref:") || element instanceof FormattedString_1.default) {
                            var r = element instanceof ReferenceService_1.ObjectReference ?
                                element :
                                this.app.resolve(ReferenceService_1.default).put(element);
                            url.query[key.startsWith("ref:") ? key : "ref:" + key] =
                                r.key;
                            continue;
                        }
                        if (typeof element !== "string" &&
                            (typeof element === "object" || Array.isArray(element))) {
                            url.query["json:" + key] = JSON.stringify(element);
                        }
                        else {
                            url.query[key] = element;
                        }
                    }
                }
            }
            for (var _i = 0, _a = this.callbacks; _i < _a.length; _i++) {
                var iterator = _a[_i];
                var r = iterator(url, options);
                if (r) {
                    return r;
                }
            }
            return this.openWindow(url, options);
        };
        /**
         * Sends signal to remove window/popup/frame, it will not immediately remove, because
         * it will identify whether it can remove or not by displaying cancellation warning. Only
         * if there is no cancellation warning or user chooses to force close, it will not remove.
         * @param id id of an element
         * @returns true if view was removed successfully
         */
        NavigationService.prototype.remove = function (view, force) {
            return __awaiter(this, void 0, void 0, function () {
                var vm, a;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            if (force) {
                                this.app.broadcast("atom-window-cancel:" + view.id, "cancelled");
                                return [2 /*return*/, true];
                            }
                            vm = view.viewModel;
                            if (!(vm && vm.cancel)) return [3 /*break*/, 2];
                            return [4 /*yield*/, vm.cancel()];
                        case 1:
                            a = _a.sent();
                            return [2 /*return*/, a];
                        case 2:
                            this.app.broadcast("atom-window-cancel:" + view.id, "cancelled");
                            return [2 /*return*/, true];
                    }
                });
            });
        };
        NavigationService.prototype.registerNavigationHook = function (callback) {
            var _this = this;
            this.callbacks.push(callback);
            return {
                dispose: function () {
                    types_1.ArrayHelper.remove(_this.callbacks, function (a) { return a === callback; });
                }
            };
        };
        return NavigationService;
    }());
    exports.NavigationService = NavigationService;
});
// Do not mock Navigation unless you want it in design time..
// Mock.mock(NavigationService, "MockNavigationService");
//# sourceMappingURL=NavigationService.js.map